/* Encyklopedie jazyka C
 *
 * Kapitola Struktury
 *
 */

/* Copyright (c) David Martinek, 2023
 * david.martinek.78 (zavináč) gmail.com
 *
 * This work is licensed under a Creative Commons
 * Attribution-NonCommercial-ShareAlike 4.0 International License.
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/
 *
 * Toto dílo je licencováno pole licence Creative Commons
 * Uveďte původ-Neužívejte dílo komerčně-Zachovejte licenci 4.0 Mezinárodní
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/deed.cs
 */

#define VERZE "3.7"
#define NAZEV "Struktury (+ unie + bitová pole)"

// TODO Prostuduj si obsah tohoto hlavičkového souboru! Jsou tam zaklínadla,
// bez kterých některé věci ve Windows nefungují správně!
// - barevný výstup
// - printg - český tisk (překódovává z UTF8 na CP852)
// - formátovací a formátové značky pro printf/scanf podle normy
// - obsluha nabídek v kapitolách
#include "pomocny-modul.h"

#include "menu.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <ctype.h>
#include <math.h>
// #include <float.h>
// #include <limits.h>


// Funkce realizující jednotlivé kapitoly
void typStruct(void);
void deklaraceADefiniceStruktury(void);
void pristupKPolozkam(void);
void inicializaceStruktury(void);
void kopieAPorovnani(void);
void nacitaniATisk(void);

void dynamickePromenne(void);
void parametrFunkce(void);
void navratovaHodnotaFunkce(void);
void strukturaSUkazatelemNaObjekt(void);
void strukturaSUkazatelemNaFunkci(void);
void zakladOOP(void);

void typUnion(void);
void bitovePole(void);


int main(void)
{
  povolBarevnyVystup();
//  prepniRychlostVypraveni(2);

  Tmenu * menu = menuVytvor(
    NADPIS(
      "=====================================================================\n"
      "Encyklopedie jazyka C - " NAZEV " (verze " VERZE ")\n"
      "(c) David Martinek, 2023\n"
      "Zároveň s programem sleduj i zdrojový text!\n"
      "=====================================================================\n"
    )
    , zobrazMod
  );

  if (menu == NULL)
  {
    printg("Není dost paměti ani pro výrobu menu. Končím program.");
    return 0;
  }

  menuPridej(menu,   0, "\n" SEKCE("Struktury obecně") , NULL);
  menuPridej(menu, 'A', "Datový typ struct\n", typStruct);
  menuPridej(menu, 'B', "Deklarace a specifikace typu\n" , deklaraceADefiniceStruktury );
  menuPridej(menu, 'C', "Přístup k položkám\n", pristupKPolozkam);
  menuPridej(menu, 'D', "Proměnné a inicializace\n", inicializaceStruktury);
  menuPridej(menu, 'E', "Kopie a porovnávání\n", kopieAPorovnani);
  menuPridej(menu, 'F', "Načítání a tisk\n", nacitaniATisk);

  menuPridej(menu,   0, "\n" SEKCE("Struktura jako Abstraktní datový typ (ADT)"), NULL);
  menuPridej(menu, 'G', "Ukazatele a dynamicky alokované proměnné\n", dynamickePromenne);
  menuPridej(menu, 'H', "Struktura jako parametr funkce\n", parametrFunkce);
  menuPridej(menu, 'I', "Struktura jako návratová hodnota funkce\n", navratovaHodnotaFunkce);
  menuPridej(menu, 'J', "Struktura s ukazatelem na dynamický objekt (lineární seznam apod.)\n", strukturaSUkazatelemNaObjekt);
  menuPridej(menu, 'K', "Struktura s ukazatelem na funkci\n", strukturaSUkazatelemNaFunkci);
  menuPridej(menu, 'L', "Struktura jako základ OOP\n", zakladOOP);// konstrukce ADT pomocí struktury

  menuPridej(menu,   0, "\n" SEKCE("Unie a bitová pole (pro hackery)"), NULL);
  menuPridej(menu, 'M', "Datový typ union\n", typUnion);
  menuPridej(menu, 'N', "Bitové pole\n", bitovePole);

  menuPridej(menu,   0, "\n" SEKCE("Nastavení programu"), NULL);
  menuPridej(menu, '0', "Nastav vyprávěcí mód\n", nastavRychlostVypraveni);

  while (menuVolba(menu) != 'Q') {}

  printg("Končím program.\n");
  menuZrus(menu);
  return 0;
}




////////////////////////////////////////////////////////////////////////////////
void typStruct(void)
{
  zacatekKapitoly("Datový typ struct");

  printh(
    "- Je to " BB("uživatelský typ") ", který si musí programátor navrhnout sám.\n"
    "- Je to " BB("heterogenní typ") ", který je tvořen položkami různých typů.\n"
    "- Je to základní nástroj strukturovaného programování a přímý předchůdce\n"
    "  typu třída z objektově orientovaného programování.\n"
    // Strukturované programování je charakteristické vytvářením podprogramů
    // zpracovávajících datové struktury - mimo jiných i typu struct.
    // Strukturované programování = podprogramy + datové struktury
    // OOP pak klade na datové struktury (třídy) ještě mnohem větší důraz.
    "\n"
  );

  if (navratNeboSekce("Uživatelský typ")) return;
  if (pokracovat())
  {
    printh(
      "- Není to zabudovaný datový typ.\n"
      "- Uživatel si jej musí sám vytvořit.\n"
      "- Proměnná typu struktura je složena z více vnitřních proměnných.\n"
      "- Vnitřním prvkům struktury se říká " BB("položky") ", " BB("složky") " nebo " BB("členy") ".\n"
      "- Položek může být libovolné množství.\n"
      "- Položky jsou pojmenované.\n"
      "- S každou vnitřní položkou jde zacházet jako se samostatnou proměnnou.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Heterogenní typ")) return;
  if (pokracovat())
  {
    printh(
      "- Na rozdíl od pole mohou být položky struktury různých typů.\n"
      "- K jednotlivým položkám se přistupuje přes název proměnné a názvy položek.\n"
      "\n"
      PODSEKCE("Ukázka deklarace typu struktura")
      "\n"
      KOD(
      "  typedef struct {\n"
      "    unsigned int id;\n"
      "    char jmeno[20+1];\n"
      "    char prijmeni[20+1];\n"
      "    float vyska;\n"
      "    unsigned int vek;\n"
      "  } Tosoba;\n"
      "  \n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Využití typu struktura")) return;
  if (pokracovat())
  {
    printh(
      "- Hlavní nástroj pro datovou abstrakci.\n"
      "- Jedna proměnná je složena z několika vnitřních proměnných -- respektive\n"
      "  více proměnných je zabaleno do jedné souhrnné proměnné.\n"
      "- Používá se pro slučování hodnot, které logicky patří k sobě a má smysl\n"
      "  s nimi v určitých situacích zacházet jako s jediným objektem.\n"
      "- Je to základ strukturovaného programování.\n"
      "- Je to základ také objektově orientovaného programování.\n"
      "  (Ano, i v jazyce C jde používat některé principy OOP.)\n"
      "\n"
      PODSEKCE("Příklad: Osoba")
      "- Je definována svým identifikačním číslem, jménem, příjmením, výškou\n"
      "  a věkem.\n"
      "- Se všemi údaji se dá pracovat jako se samostatnými proměnnými, ale\n"
      "  když chci mít funkci, která zpracuje jednu osobu, musí mít 5 parametrů.\n"
      "- Když chci vytvořit pole osob, je jednodušší vytvořit jedno pole osob\n"
      "  než 5 polí s jednotlivými vlastnostmi.\n"
      "- Obecně je o osobě snazší uvažovat jako o jedné entitě než jako\n"
      "  o 5 samostatných vlastnostech.\n"
      "- Je to také bezpečnější - hrozí menší riziko chyb, jako jsou záměny\n"
      "  a nekonzistence.\n"
    );
  }

  konecKapitoly(NULL);
}




////////////////////////////////////////////////////////////////////////////////
void deklaraceADefiniceStruktury(void)
{
  zacatekKapitoly("Deklarace a specifikace typu struktura");

  printh(
    "- Struktura je uživatelský typ.\n"
    "- Uživatel si jej musí sám specifikovat.\n"
    "- Dvojslovný název typu " KK("struct název") " se obvykle nahrazuje jednoslovným\n"
    "  názvem vytvořeným pomocí specifikátoru " KK("typedef") ".\n"
    "\n"
  );

//  // špatně
//  typedef struct {
//    int minimum = 0;
//    int maximum = 50;
//  } Tinterval;

  // správně
  typedef struct {
    int minimum;
    int maximum;
  } Tinterval;


  // Definice proměnné typu Tinterval
  // Ukázka použití inicializátoru proměnné typu struct
  Tinterval a = {
    .minimum = 0,  // inicializace položky a.minimum
    .maximum = 50, // inicializace položky a.maximum
  };

  if (navratNeboSekce("Terminologická poznámka")) return;
  if (pokracovat())
  {
    printh(
      "- Rozlišuj typ struktura a proměnnou typu struktura!\n"
      "- Typ popisuje, jak má vypadat proměnná.\n"
      "- O proměnné typu struktura můžeme mluvit jako o instanci nebo dokonce\n"
      "  o objektu tohoto typu.\n"
      "- Začátečníci to často nerozlišují a pokoušejí se přiřadit hodnoty\n"
      "  vnitřním položkám už při deklaraci typu, což je samozřejmě nesmysl.\n"
      "\n"
      "- Při specifikaci typu struktura její položky inicializovat nejde!\n"
      "- Specifikace typu se obvykle vyskytuje mimo těla funkcí.\n"
      "\n"
      KOD(
      "  // špatně                     |    // správně\n"
      "  typedef struct {              |    typedef struct {\n"
      "    int minimum = 0;  // !!!    |      int minimum;\n"
      "    int maximum = 50; // !!!    |      int maximum;\n"
      "  } Tinterval;                  |    } Tinterval;\n"
      )
      "\n"
      "- Inicializovat jde proměnnou, ne typ. Typ musí být hotov před výrobou proměnné!\n"
      "- Proměnné (objekty) se obvykle vytvářejí a inicializují uvnitř funkcí.\n"
      "\n"
      KOD(
      "  // Definice proměnné typu Tinterval\n"
      "  // Ukázka použití inicializátoru proměnné typu struct\n"
      "  Tinterval a = {\n"
      "    .minimum = 0,  // inicializace položky a.minimum\n"
      "    .maximum = 50, // inicializace položky a.maximum\n"
      "  };\n"
      )
      "\n"
      "  => a.minimum: %d\n"
      "  => a.maximum: %d\n"
      "\n"
      , a.minimum
      , a.maximum
    );
  }

  if (navratNeboSekce("Specifikace struktury (struct nazev)")) return;
  if (pokracovat())
  {
    printh(
      "- Původně navržený způsob není moc pohodlný pro použití - nedoporučuji.\n"
      "\n"
      KOD(
      "  struct interval {\n"
      "    int minimum,\n"
      "    int maximum,\n"
      "  };\n"
      )
      "\n"
      "- Název nového typu je dvouslovný: " KK("struct interval") ".\n"
      "- Deklarace proměnné vypadá takto:\n"
      "\n"
      KOD(
      "  struct interval a;\n"
      )
      "\n"
      "- Deklarace proměnné typu ukazatel na " KK("struct interval") " vypadá takto:\n"
      "\n"
      KOD(
      "  struct interval * b;\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Specifikace struktury (typedef)")) return;
  if (pokracovat())
  {
    printh(
      "- Použijeme specifikátor " KK("typedef") " pro přejmenování na jednoslovný název.\n"
      "\n"
      PODSEKCE("Konvence")
      "- Nový název typu bude začínat písmenem T.\n"
      "- Název za klíčovým slovem struct začíná podtržítkem, aby se nepletl s jinými\n"
      "  identifikátory.\n"
      "- Název za klíčovým slovem struct teď není povinný, ale může se hodit.\n"
      "\n"
      KOD(
      "  // správně                    |    // taky správně \n"
      "  typedef struct _interval {    |    typedef struct {\n"
      "    int minimum;                |      int minimum;\n"
      "    int maximum;                |      int maximum;\n"
      "  } Tinterval;                  |    } Tinterval;\n"
      )
      "\n"
      "- Název nového typu je jednoslovný: " KK("Tinterval") ".\n"
      "- Syntakticky se s ním zachází stejně jako s jinými jednoslovnými názvy typů.\n"
      "- Deklarace proměnné vypadá takto:\n"
      "\n"
      KOD(
      "  Tinterval a;\n"
      )
      "\n"
      "- Deklarace proměnné typu ukazatel na " KK("Tinterval") " vypadá takto:\n"
      "\n"
      KOD(
      "  Tinterval * b;\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Specifikace struktury (typedef v.2)")) return;
  if (pokracovat())
  {
    printh(
      "- Specifikátor " KK("typedef") " použijeme jinak.\n"
      "- Přejmenování a deklarace typu struktura bude odděleno.\n"
      "- Tento způsob se používá v modulech a knihovnách.\n"
      "- Pokud první část bude v hlavičkovém a druhá v implementačním souboru,\n"
      "  budou položky struktury mimo knihovnu nedostupné. S typem pak půjde\n"
      "  pracovat výhradně přes ukazatel a uživatel neuvidí, jaké položky\n"
      "  proměnná obsahuje - nemůže je tedy ani nijak poničit.\n"
      "- Takto je vnitřně udělán například typ " KK("FILE*") " z knihovny stdio.h.\n"
      "- Nyní nelze vynechat identifikátor za klíčovým slovem struct.\n"
      "\n"
      KOD(
      "  // Tohle bude typicky v souboru interval.h.\n"
      "  // Odteď jde používat typ Tinterval.\n"
      "  typedef struct _interval Tinterval;\n"
      "\n"
      "  // Tohle bude typicky v souboru interval.c.\n"
      "  // Odložená specifikace typu struct _interval\n"
      "  struct _interval  {\n"
      "    int minimum;\n"
      "    int maximum;\n"
      "  };\n"
      // Poznámka: Když to zůstane pohromadě v jednom hlavičkovém souboru, tak
      // položky struktury samozřejmě budou viditelné všude, kde se ten hlavičkový
      // soubor použije, respektive kde bude ta deklarace viditelná.
      )
      "\n"
      "- Název nového typu je opět jednoslovný: " KK("Tinterval") ".\n"
      "- Deklarace proměnné vypadá stejně jako u varianty č. 1:\n"
      "\n"
      KOD(
      "  Tinterval a;\n"
      )
      "\n"
      "- Bude-li odložená specifikace typu " KK("struct interval") " v samostatném\n"
      "  souboru, bude možné vytvořit jen proměnnou typu ukazatel na " KK("Tinterval") ".\n"
      "\n"
      KOD(
      "  Tinterval* b;\n"
      )
      "\n"
      "- Poznámka: V modulu interval.c samozřejmě s vnitřními položkami pracovat\n"
      "  půjde - tam viditelné jsou. Přínosem je, že mimo tento soubor vidět\n"
      "  nejdou, proto je uživatel tohoto modulu nemůže omylem poškodit.\n"
      "\n"
    );
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////
void pristupKPolozkam(void)
{
  typedef struct {
    int id;
    char nazev[20+1];
    float hodnota;
  } Tprvek;

  Tprvek p; // p je proměnná obsahující 3 položky
  p.id = 1; // položka id může vystupovat jako samostatná proměnná
  strcpy(p.nazev, "police");
  p.hodnota = 959.9;

  zacatekKapitoly("Přístup k položkám proměnné typu struktura");

  printh(
    "- Každá položka ve struktuře může vystupovat jako samostatná proměnná.\n"
    "- S proměnnou typu struktura jde pracovat jako s celkem, ale i po položkách.\n"
    "\n"
    KOD(
    "  typedef struct {\n"
    "    int id;\n"
    "    char nazev[20+1];\n"
    "    float hodnota;\n"
    "  } Tprvek;\n"
    )
    "\n"
    "\n"
  );

  if (navratNeboSekce("Tečkový operátor")) return;
  if (pokracovat())
  {
    printh(
      "- Pro přístup k položce nějaké proměnné je potřeba název proměnné\n"
      "  a název položky.\n"
      "- Používá se takzvaný tečkový operátor mezi názvem proměnné a položky.\n"
      "\n"
      KOD(
      "  Tprvek p; // p je proměnná obsahující 3 položky\n"
      "  p.id = 1; // položka id může vystupovat jako samostatná proměnná\n"
      "  strcpy(p.nazev, \"police\");  // pozn.: pole nejde kopírovat přiřazením\n"
      "  p.hodnota = 959.9;\n"
      )
      "\n"
      "  => p.id: %d\n"
      "  => p.nazev: %s\n"
      "  => p.hodnota: %g\n"
      "\n"
      , p.id
      , p.nazev
      , p.hodnota
    );
  }

  if (navratNeboSekce("Šipkový operátor (->)")) return;
  if (pokracovat())
  {
    Tprvek *q = &p;      // adresa skutečné proměnné
  //  *q.hodnota = 800;    // chyba!
    (*q).hodnota = 800;  // správně
    q->id = 5;

    printh(
      "- Pokud je proměnná ukazatelem na proměnnou typu struktura, je přístup\n"
      "  k položkám možný pomocí kombinace dereferenčního a tečkového operátoru.\n"
      "- V tom případě se ale musí použít závorky, protože tečkový operátor\n"
      "  má vyšší prioritu než dereferenční.\n"
      "\n"
      KOD(
      "  Tprvek *q = &p;      // adresa skutečné proměnné\n"
      "  *q.hodnota = 800;    // chyba!\n"
      "  (*q).hodnota = 800;  // správně\n"
      )
      "\n"
      "- V praxi se tohle nepoužívá. Stejnou funkci má šipkový operátor.\n"
      "- Šipkový operátor je pohodlnější.\n"
      "- Používá se i v C++.\n"
      "\n"
      KOD(
      "  q->id = 5;\n"
      )
      "\n"
      "Výsledná hodnota\n"
      "  => q->id: %d\n"
      "  => q->nazev: %s\n"
      "  => q->hodnota: %g\n"
      "\n"
      , q->id
      , q->nazev
      , q->hodnota
    );
  }

  if (navratNeboSekce("Častá chyba")) return;
  if (pokracovat())
  {
//    Tprvek.hodnota = 1.5;  // Nesmysl!
//    Tprvek->hodnota = 1.5; // Taky ptákovina

    printh(
      "- Častou chybou je pokus o přístup k položkám přes název typu.\n"
      "\n"
      KOD(
      "  Tprvek.hodnota = 1.5;  // Nesmysl!\n"
      "  Tprvek->hodnota = 1.5; // Taky ptákovina!\n"
      )
      "\n"
      "- Obojí je syntaktickou chybou.\n"
      "- Typ si pamatuje překladač, ale v paměti programu se nikde nenachází.\n"
      "- Podle typu je potřeba vyrobit proměnnou, se kterou jde dále pracovat.\n"
      "- Odkazované položky se mohou nacházet jen uvnitř konkrétní proměnné.\n"
      "\n"
    );
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////

typedef struct {
  float x, y;
} Tsouradnice;

typedef struct
{
  char nazev[10+1];
  Tsouradnice stred;
  float vyska, sirka;
} Tobjekt;


/** Vytiskne objekt do textového řetězce pomocí sprintf.
 *
 * Tato funkce se hodí pro případy, kdy je potřeba s vytištěnou hodnotou ještě
 * dále pracovat.
 *
 * \param str char* Pole znaků, které musí mít dostatek místa pro výsledek.
 * \param obj const Tobjekt Objekt, který se bude tisknout.
 * \return char* Vrací ukazatel na začátek pole str. To se hodí, protože pak
 * lze tuto funkci použít třeba jako parametr funkce printf. Pozor! V takovém
 * případě je třeba být opatrný při vícenásobném použití.
 *
 * Tohle je v pohodě:
 * printf("objekt: %s", objektNaString(str, objekt));
 *
 * Tohle vytiskne dvakrát jeden z objektů!
 * printf("o1: %s, o2: %s", objektNaString(str, o1), objektNaString(str, o2));
 *
 * Takto je to správně:
 * printf("o1: %s, o2: %s", objektNaString(str1, o1), objektNaString(str2, o2));
 */
char * objektNaString(char *str, Tobjekt obj)
{
  sprintf(
    str,
    "{ .nazev:%s, .stred.x:%.2f, .stred.y:%.2f, .vyska:%.2f, .sirka:%.2f }"
    , obj.nazev
    , obj.stred.x
    , obj.stred.y
    , obj.vyska
    , obj.sirka
  );
  return str;
}


void inicializaceStruktury(void)
{
  char str1[100];
  char str2[100];

  zacatekKapitoly("Inicializace proměnné typu struktura");

  printh(
    "- Inicializovat jde proměnné, nikoli typy - viz předchozí kapitola.\n"
    "- Každá položka struktury je samostatná proměnná - jde ji inicializovat\n"
    "  samostatně.\n"
    "- Při definici proměnné lze použít inicializátor - pohodlnější způsob.\n"
    "\n"
    "\n"
  );

  if (navratNeboSekce("Ukázkové typy")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  typedef struct {\n"
      "    float x, y;\n"
      "  } Tsouradnice;\n"
      "  \n"
      "  typedef struct\n"
      "  {\n"
      "    char nazev[10+1];\n"
      "    Tsouradnice stred;\n"
      "    float vyska, sirka;\n"
      "  } Tobjekt;\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Inicializace po položkách")) return;
  if (pokracovat())
  {
    Tobjekt obdelnik;
    strcpy(obdelnik.nazev, "obdelnik");
    obdelnik.stred.x = 65;
    obdelnik.stred.y = 15;
    obdelnik.vyska = 5;
    obdelnik.sirka = 15;

    printh(
      "- Tento způsob je nejpracnější.\n"
      "- Položky typu pole (tedy ani textový řetězec) nelze nastavit přiřazením!\n"
      "\n"
      KOD(
      "  Tobjekt obdelnik;\n"
      "  strcpy(obdelnik.nazev, \"obdelnik\");\n"
      "  obdelnik.stred.x = 65;\n"
      "  obdelnik.stred.y = 15;\n"
      "  obdelnik.vyska = 5;\n"
      "  obdelnik.sirka = 15;\n"
      )
      "\n"
      "  => obdelnik: %s\n"
      "\n"
      , objektNaString(str1, obdelnik)
    );
  }

  if (navratNeboSekce("Inicializátor")) return;
  if (pokracovat())
  {
    printh(
      "- Inicializátor je konstrukce, kterou lze použít při definici proměnné.\n"
      "- Inicializátor lze použít výhradně pro složené typy, tedy struktury a pole.\n"
      "- Obecný tvar inicializátoru tvoří inicializační přiřazení a složené\n"
      "  závorky, uvnitř nich jsou inicializační hodnoty položek proměnné.\n"
      "\n"
      KOD(
      "  Typ proměnná = { inicializace prvků };\n"
      )
      "\n"
      "- Není povinné inicializovat všechny položky proměnné.\n"
      "- Pokud je použit inicializátor a je inicializována jakákoli položka\n"
      "  libovolnou hodnotou, pak všechny ostatní položky jsou automaticky\n"
      "  inicializovány nulovou hodnotou nebo jejím ekvivalentem.\n"
      "\n"
    );
  }


  if (navratNeboSekce("Anonymní hodnota")) return;
  if (pokracovat())
  {
    printh(
      "- Inicializátor lze použít i pro konstrukci anonymních hodnot.\n"
      "- Je však nutné před ním použít operátor přetypování, aby měl překladač\n"
      "  informaci o tom, jak má hodnota vypadat uvnitř.\n"
      "- Funguje v novějších verzích jazyka C (C99, C11), které umí kopírovat\n"
      "  obsah proměnných typu struktura přiřazovacím operátorem.\n"
      "\n"
      KOD(
      "  Typ proměnná;\n"
      "  proměnná = { inicializace prvků }; // tohle je chyba!\n"
      "  proměnná = (Typ) { inicializace prvků }; // tohle je správně\n"
      )
      "\n"
      "- Anonymní hodnotu lze použít i jako argument funkce, která má parametr\n"
      "  typu struktura. Není potřeba vyrábět a inicializovat proměnnou.\n"
      "\n"
      KOD(
      "  zpracujStrukturu( (Typ) { inicializace prvků });\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Inicializátor bez názvů položek")) return;
  if (pokracovat())
  {
    Tobjekt ctverec = {"ctverec", {25, 30}, 5, 5 };
    Tobjekt trojuhelnik;
    trojuhelnik = (Tobjekt) {"trojuhelnik", {67, 90}, 20, 10 };

    printh(
      "- Tato varianta funguje i v původní verzi jazyka C.\n"
      "- Inicializátor obsahuje pouze výčet hodnot, oddělených čárkou.\n"
      "- Nejsou specifikovány názvy položek.\n"
      "- Pořadí hodnot v inicializátoru musí odpovídat pořadí položek při specifikaci\n"
      "  typu struktura.\n"
      "- Na položky strukturovaných typů (struct, pole) se používá vnořený\n"
      "  inicializátor.\n"
      "\n"
      KOD(
      "  Tobjekt ctverec = {\"ctverec\", {25, 30}, 5, 5 };\n"
      "  Tobjekt trojuhelnik;\n"
      "  trojuhelnik = (Tobjekt) {\"trojuhelnik\", {67, 90}, 20, 10 };\n"
      )
      "\n"
      "  => ctverec: %s\n"
      "  => trojuhelnik: %s\n"
      "\n"
      , objektNaString(str1, ctverec)
      , objektNaString(str2, trojuhelnik)
    );
  }

  if (navratNeboSekce("Inicializátor s názvy položek")) return;
  if (pokracovat())
  {
    Tobjekt objX = {.sirka=2, .stred={.x=125}, .vyska=10};
    Tobjekt objY;
    objY = (Tobjekt) {.vyska=5, .stred={.y=70, .x=55}, .sirka=10, .nazev="objY"};


    printh(
      "- Tato varianta funguje v novějších verzích jazyka C, jako C99, C11, ...\n"
      "- Není potřeba dodržet pořadí položek jako při specifikaci typu.\n"
      "- Mohou být vynechány i první položky.\n"
      "\n"
      KOD(
      "  Tobjekt objX = {.sirka=2, .stred={.x=125}, .vyska=10};\n"
      "  Tobjekt objY;\n"
      "  objY = (Tobjekt) {.vyska=5, .stred={.y=70, .x=55}, .sirka=10, .nazev=\"objY\"};\n"
      )
      "\n"
      "  => objX: %s\n"
      "  => objY: %s\n"
      "\n"
      , objektNaString(str1, objX)
      , objektNaString(str2, objY)
    );
  }

  if (navratNeboSekce("Inicializátor s vynechanými položkami")) return;
  if (pokracovat())
  {
    Tobjekt neinicializovany;
    Tobjekt nulovy = {.sirka = 0};

    neinicializovany.nazev[10] = '\0'; // hack, aby to náhodou nehavarovalo při tisku

    printh(
      "- U všech inicializátorů platí, že vynechané položky se automaticky nastaví\n"
      "  na hodnotu 0.\n"
      "- Platí to pouze, když je inicializátor použit.\n"
      "- Proměnná bez inicializace bude mít náhodné hodnoty položek.\n"
      "- Pozor! Často se může zdát, že neinicializovaný objekt má nastaveny nulové\n"
      "  hodnoty položek. To ale není spolehlivé. Často je to dáno tím, že program\n"
      "  byl přeložen v IDE v ladícím módu a paměť pro něj byla dopředu \"učesána\".\n"
      "\n"
      KOD(
      "  Tobjekt neinicializovany;\n"
      "  Tobjekt nulovy = {.sirka = 0};\n"
      )
      "\n"
      "  => neinicializovany: %s\n"
      "  => nulovy: %s\n"
      "\n"
      , objektNaString(str1, neinicializovany)
      , objektNaString(str2, nulovy)
    );
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////

/** Porovná objekty a, b podle položek specifikovaných v parametru poradi.
 *
 * Typ Tobjekt má 5 položek. Jejich pořadí nechť je:
 * 1 - nazev
 * 2 - stred.x
 * 3 - stred.y
 * 4 - vyska
 * 5 - sirka
 *
 * Parametr poradi udává, podle kterých položek a v jakém pořadí se mají
 * porovnávat. Hodnota poradi="541" znamená, že se bude porovnávat nejprve
 * podle položky sirka. Pokud jsou šířky stejné, bude porovnávat podle položky
 * vyska. Pokud jsou i tyto hodnoty stejné, bude se porovnávat podle položky
 * nazev. Položky stred.x a stred.y se vůbec nebudou brát v potaz.
 *
 * Poznámka: Další možností je vyrobit typ ukazatel na funkci a vyrobit několik
 * funkcí, které budou porovnávat objekty podle různých kritérií. Pak je možné
 * je předávat do jiných algoritmů jako parametr funkce - viz kapitolu Funkce.
 *
 * \return int Vrací 0, když jsou objekty stejné,
 *                  <0, když je a < b,
 *                  >0, když je a > b.
 */
int porovnejObjektyPodleKlicu(Tobjekt a, Tobjekt b, char * klice)
{
  int vysledek = 0;
  for (int i = 0; vysledek == 0 && klice[i] != '\0'; ++i)
  {
    switch (klice[i])
    {
      case '1': vysledek = strcmp(a.nazev, b.nazev); break;
      case '2': vysledek = a.stred.x - b.stred.x;    break;
      case '3': vysledek = a.stred.y - b.stred.y;    break;
      case '4': vysledek = a.vyska - b.vyska;        break;
      case '5': vysledek = a.sirka - b.sirka;        break;

      default:  return vysledek; break;
    }
  }

  return vysledek;
}


void kopieAPorovnani(void)
{
  char str1[100];
  char str2[100];

  Tobjekt a = {.nazev="objZ", .stred={.x=55, .y=70}, .vyska=5, .sirka=10 };
  Tobjekt b = a;

  zacatekKapitoly("Operace nad proměnnými");

  printh(
    "- Nad proměnnou typu struktura je často potřeba provádět operace jako\n"
    "  kopie a porovnávání.\n"
    "- Mělkou kopii jde udělat přímo prostředky jazyka.\n"
    "- Hluboká kopie a porovnání jsou netriviální operace. Uživatel si je musí\n"
    "  vytvořit podle svých potřeb jako podprogramy.\n"
    "\n"
  );

  if (navratNeboSekce("Mělká kopie hodnot")) return;
  if (pokracovat())
  {
    printh(
      "- Hodnoty typu struktura jde v jazyce C (C99 a novější) kopírovat přiřazením.\n"
      "- Jde ovšem kopírovat pouze proměnné stejného typu.\n"
      "-- I kdyby měly dva typy struktura totožné položky, půjde o rozdílné typy!\n"
      "- Přiřazením dojde k tzv. mělké kopii vnitřních položek.\n"
      "- Kopírují se i položky typu pole (s pevnou velikostí).\n"
      "- U položek typu ukazatel se kopíruje jen adresa, ne odkazovaná paměť.\n"
      "- Pozor u textových řetězců - záleží, jak jsou tyto položky deklarovány.\n"
      "-- Jde-li o pole s pevnou velikostí (" KK("char text[10+1]") "), kopíruje se\n"
      "   celé pole.\n"
      "-- Jde-li jen o ukazatel na textový řetězec (" KK("char *text") "), kopíruje se\n"
      "   jen tento ukazatel. Skutečný řetězec zůstává původní.\n"
      "\n"
      KOD(
      "  Tobjekt a = {.nazev=\"objZ\", .stred={.x=55, .y=70}, .vyska=5, .sirka=10 };\n"
      "  Tobjekt b = a;\n"
      )
      "\n"
      "  => a: %s\n"
      "  => b: %s\n"
      "\n"
      , objektNaString(str1, a)
      , objektNaString(str2, b)
    );
  }

  if (navratNeboSekce("Poznámka k podprogramům")) return;
  if (pokracovat())
  {
    printh(
      "- U parametrů předávaných hodnotou dojde při volání skutečně ke kopii\n"
      "  celé proměnné, tedy všech položek.\n"
      "- Někdy může jít o poměrně hodně dat, zvláště když jsou položky typu pole.\n"
      "- Často se proto předávají parametry typu struktura odkazem - je to efektivnější.\n"
      "-- Uvnitř funkce se pak místo tečkového používá šipkový operátor.\n"
      "-- Pokud nechceš takový parametr změnit omylem, lze to ošetřit přidáním\n"
      "   klíčového slova " KK("const") " před deklaraci parametru.\n"
      "\n"
      KOD(
      "  void vypisObjekt(const Tobjekt *o);\n"
      )
      "\n"
      "- Funkce mohou vracet hodnoty typu struktura, protože výsledek lze\n"
      "  na rozdíl od polí přiřadit (zkopírovat) do jiné proměnné.\n"
      "\n"
      KOD(
      "  Tobjekt vytvorObjekt(char *nazev, Tsouradnice stred, int vyska, int sirka);\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Porovnávání hodnot")) return;
  if (pokracovat())
  {
    printh(
      "- Hodnoty celých struktur nelze porovnávat porovnávacími operátory.\n"
      "- Porovnávat se dají pouze položky jednoduchých typů.\n"
      "- Pro porovnání celých objektů je potřeba vytvořit porovnávací funkci.\n"
      "-- Ideálně takovou, která funguje podobně jako " KK("strcmp") ", tedy\n"
      "   vrací nulu, když jsou objekty stejné, hodnotu <0, když je první objekt\n"
      "   menší a hodnotu >0, když je první objekt větší než druhý.\n"
      "\n"
      KOD(
      "  int porovnejObjekty(Tobjekt a, Tobjekt b);\n"
      "\n"
      "  // U velkých objektů raději takto.\n"
      "  int porovnejObjekty(const Tobjekt *a, const Tobjekt *b);\n"
      )
      "\n"
      "- Porovnáním výsledku s nulou lze realizovat všech 6 porovnávacích operátorů.\n"
      "\n"
      "- Porovnávání podle více klíčů lze realizovat například tak, že přidáme\n"
      "  parametr udávající pořadí, v jakém se mají položky porovnávat.\n"
      "\n"
      KOD(
      "  int porovnejObjektyPodleKlicu(Tobjekt a, Tobjekt b, char * klice);\n"
      )
      "\n"
      "- Použití může vypadat takto:\n"
      "\n"
      KOD(
      "  if (porovnejObjektyPodleKlicu(a, b, \"541\") == 0)\n"
      "    printf(\"Objekty a, b maji stejnou sirku, vysku i nazev.\\n\");\n"
      "  else\n"
      "    printf(\"Objekty a, b jsou ruzne.\\n\");\n"
      )
      "\n"
      "  => "
    );

    if (porovnejObjektyPodleKlicu(a, b, "541") == 0)
      printf("Objekty a, b maji stejnou sirku, vysku i nazev.\n");
    else
      printf("Objekty a, b jsou ruzne.\n");

    printf("\n");
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////

/** Načte ze vstupu jeden objekt.
 *
 * Formát vstupních dat:
 * název, střed.x, střed.y, výška, šířka
 *
 * Oddělovačem jsou čárky a libovolné počty mezer.
 *
 * \param in FILE*    Vstupní soubor nebo stream
 * \param o Tobjekt*  Cílový objekt předávaný odkazem.
 * \return int Vrací analogii toho, co scanf.
 *             Vrací 1, když se celý objekt povede přečíst,
 *                   0, když se objekt přečíst nepovedlo,
 *                   EOF, když došlo ke čtení za koncem souboru.
 */
int nactiObjekt(FILE *in, Tobjekt *o)
{
  // Mezery ve formátovém řetězci znamenají místa, kde se na vstupu mohou
  // vyskytovat bílé znaky, které budou ignorovány.
  // Mezera na začátku je důležitá, protože přeskočí mezery a konce řádků zbylé
  // od předchozího zadání.
  // Na konci by mezera být neměla, protože to vadí při interaktivním zadávání
  // - čtení by pak končilo až zadáním nějakého nebílého znaku.
  // Všimni si, že první načtený název nebude začínat nikdy mezerou, ale může
  // obsahovat mezery. Pokud budou mezery před první čárkou, budou součástí
  // načteného řetězce, proto je obvykle potřeba takový řetězec ještě ostřihnout.
  int znacek = fscanf(in, " %10[^,] , %f , %f , %f , %f"
                , o->nazev, &o->stred.x, &o->stred.y, &o->vyska, &o->sirka);
  if (znacek <= 0) return znacek; // 0, EOF
  if (znacek == 5) return 1;      // 5 přečtených značek => 1 objekt
  return 0;                       // 1--4 přečtené značky => chyba, 0 načtených objektů
}


/** Vytiskne jeden objekt na výstup.
 *
 * \param vystup File* Výstupní soubor nebo stream.
 * \param o const Tobjekt* Objekt, který se bude tisknout.
 * \return int Vrací počet vytištěných znaků.
 */
int tiskniObjekt(FILE *vystup, const Tobjekt *o)
{
  return fprintf(vystup,
    "%s, %.2f, %.2f, %.2f, %.2f\n"
    , o->nazev, o->stred.x, o->stred.y, o->vyska, o->sirka
  );
}


void nacitaniATisk(void)
{
  char str[100];

  zacatekKapitoly("Načítání a tisk hodnot typu struktura");

  printh(
    "- Funkce " KK("scanf") " a " KK("printf") " nemohou mít značky pro práci s uživatelsky\n"
    "  vytvořenými typy.\n"
    "- Tyto operace se musí dělat ručně po složkách.\n"
    "\n"
  );

  Tobjekt o;

  if (navratNeboSekce("Načítání hodnot")) return;
  if (pokracovat())
  {
    printh(
      "- Funkce " KK("scanf") " a její varianty neumí načítat struktury přímo, protože\n"
      "  jde o uživatelský typ.\n"
      "- Je potřeba je číst po složkách jednoduchých typů.\n"
      "- Ideální je vytvořit si pro načítání funkci, která půjde použít podobně\n"
      "  jako standardní funkce používané pro čtení ze vstupu.\n"
      "\n"
      KOD(
      "  int nactiObjekt(FILE *in, Tobjekt *o);\n"
      "...\n"
      "  printf(\"Zadej objekt ve formatu: nazev, x, y, vyska, sirka\\n\");\n"
      "  Tobjekt o;\n"
      "  int kod = nactiObjekt(stdin, &o);\n"
      )
      "\n"
    );

    printf("Zadej objekt ve formatu: nazev, x, y, vyska, sirka\n");
    int kod = nactiObjekt(stdin, &o);

    pozerKonecRadku();

    printh(
      "\n"
      "  => o: %s\n"
      "  => kod: %d\n"
      "\n"
      , objektNaString(str, o)
      , kod
    );
  }

  if (navratNeboSekce("Tisk hodnot")) return;
  if (pokracovat())
  {
    printh(
      "- Funkce " KK("printf") " opět neumí tisknout struktury přímo.\n"
      "- Je potřeba tisknout po složkách jednoduchých typů.\n"
      "- Opět je dobré vytvořit si funkci, která objekt vytiskne. \n"
      "\n"
      "- Taková funkce rovněž umožní tisknout hodnoty konzistentně a ve stejném\n"
      "  formátu, jaký očekáváme na vstupu.\n"
      "- Aby to bylo efektivnější, je dobré předávat velké struktury odkazem.\n"
      // Poznámka: V reálném programu zřejmě budou objekty alokovány dynamicky
      // a proměnné budou ukazatele na strukturu. Z toho důvodu při předávání
      // odkazem vymizí potřeba používat referenční operátor.
      "\n"
      KOD(
      "  void tiskniObjekt(File *vystup, const Tobjekt *o);\n"
      "...\n"
      "  // tisk objektu přečteného před chvílí\n"
      "  tiskniObjekt(stdout, &o);\n"
      )
      "\n"
      "  => "
    );

    // tisk objektu přečteného před chvílí
    tiskniObjekt(stdout, &o);

    printh(
      "\n"
      "- Někdy může být užitečné tisknout textový řetězec do pole znaků, podobně\n"
      "  jako to dělá funkce " KK("sprintf") ".\n"
      "- Výsledný textový řetězec je pak možné v programu dále používat.\n"
      "\n"
      KOD(
      "  char * objektNaString(char *str, Tobjekt obj);\n"
      )
      "\n"
    );
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////

/** Alokuje a inicializuje nový objekt.
 *
 * \param nazev[] char Hodnota položky nazev.
 * \param x int Hodnota položky stred.x.
 * \param y int Hodnota položky stred.y.
 * \param vyska int Hodnota položky vyska.
 * \param sirka int Hodnota položky sirka.
 * \return Tobjekt* Vrací ukazatel na nový objekt nebo NULL.
 */
Tobjekt * novyObjekt(char nazev[], int x, int y, int vyska, int sirka)
{
  Tobjekt *o = malloc(sizeof(Tobjekt));
  if (o != NULL)
  {
    //o->nazev = nazev, //Tohle nejde.

//    *o = (Tobjekt) {
//      .nazev = nazev, // Tohle taky nejde
//      .stred = {x, y},
//      .vyska = vyska,
//      .sirka = sirka,
//    };

    // Naštěstí je tohle potřeba udělat jen jednou.
    strcpy(o->nazev, nazev);
    o->stred.x = x;
    o->stred.y = y;
    o->vyska = vyska;
    o->sirka = sirka;
  }
  return o; // Vrací ukazatel na nový objekt nebo NULL.
}


void dynamickePromenne(void)
{
  char str[100];
  Tobjekt *o;

  zacatekKapitoly("Ukazatele a dynamicky alokované proměnné");

  printh(
    "- Struktury jsou poměrně velké proměnné.\n"
    "- Často jsou základem tabulek a dynamických datových struktur.\n"
    "- Proto se často alokují dynamicky na hromadě (heap) a pracuje se s nimi\n"
    "  přes ukazatele.\n"
    "\n"
  );

  if (navratNeboSekce("Ukazatel na strukturu")) return;
  if (pokracovat())
  {
    printh(
      "- Vytváří se jako jakýkoli jiný typový ukazatel.\n"
      "- Použijeme-li náš dříve vytvořený typ " KK("Tobjekt") ", pak se proměnná\n"
      "  typu ukazatel na takovou strukturu vytvoří takto:\n"
      "\n"
      KOD(
      "  Tobjekt * o;\n"
      )
      "\n"
      "- Pozor! Tato proměnná je teď neinicializovaný ukazatel, nikoli struktura!\n"
      "- Zatím nemá smysl pracovat s vnitřními položkami struktury, protože tato\n"
      "  proměnná zatím na žádnou konkrétní proměnnou typu struktura neodkazuje.\n"
      "\n"
      "- Je možné sem přiřadit adresu existujícího objektu nebo vyrobit nový\n"
      "  pomocí dynamické alokace funkcí " KK("malloc") " (viz dále).\n"
      "\n"
      "- Položky struktury odkazované přes ukazatel jsou přístupné přes šipkový\n"
      "  operátor (viz kapitola Přístup k položkám).\n"
      "\n"
      KOD(
      "  o->vyska = 10; // Chyba! Zatím neinicializovaný ukazatel. Havaruje!\n"
      )
      "\n"
      "- Proměnná o je jen adresa někam do paměti. Nyní neinicializovaná!\n"
      "\n"
    );

// Tady vypínám varování/chybové hlášení, protože tuhle chybu chci ukázat.
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
    printh(
      "  => o: %p\n"
      "\n"
      , o // Varování překladače: neinicializovaná proměnná
    );
#pragma GCC diagnostic pop
  }

  o = malloc(sizeof(Tobjekt));
  if (o == NULL)
  {
    printf("Alokace se nepovedla! Končím\n");
    return;
  }
  o->nazev[10] = '\0'; // hack, aby to náhodou nehavarovalo při tisku

  if (navratNeboSekce("Objekt alokovaný dynamicky na hromadě (heap)")) return;
  if (pokracovat())
  {
    printh(
      "- Objekt jde vyrobit dynamicky v oblasti paměti nazývané hromada (heap).\n"
      "- Používá se k tomu funkce " KK("malloc") " ze stdlib.h.\n"
      "- Dá se použít i " KK("calloc") ", která navíc vynuluje všechny alokované\n"
      "  bajty. (Ne vždy to chceme, protože to zabírá čas.)\n"
      "- Musí se při tom použít operátor " KK("sizeof") ", který vrátí počet bajtů\n"
      "  potřebných pro proměnnou daného typu. (Více viz zdrojový text.)\n"
      // Pozor! Velikost struktury není prostým součtem velikostí jejích položek.
      // Překladač může kvůli efektivitě přidávat mezi položky prázdné bajty,
      // aby byly začátky každé položky zarovnané na 2 nebo 4 bajty (záleží na
      // architektuře procesoru. Z tohoto důvodu není dobrý nápad ukládat
      // struktury do binárních souborů přímo. Překladače sice poskytují nástroje,
      // jak udělat tzv. sbalené struktury (např. GCC používá konstrukci
      // typedef struct __attribute__((__packed__)) MojeSbalenáStruktura;), ale
      // při práci se soubory je lepší si vytvořit speciální funkce pro čtení
      // zápis, které budou ukládat a číst po složkách v předem dohodnutém formátu
      // a pořadí.
      "- Funkce " KK("malloc") " vrací ukazatel na nově vyrobený objekt nebo NULL, když\n"
      "  není k dispozici dostatek paměti.\n"
      "- Vždy je potřeba hlídat, zda se alokace povedla, jinak to vede k havárii.\n"
      "\n"
      KOD(
      "  o = malloc(sizeof(Tobjekt));\n"
      "  // Pokud se alokace se nepovedla, končím...\n"
      "  if (o == NULL) return;\n"
      )
      "\n"
      "- Objekt je nyní alokovaný ale neinicializovaný, položky mají náhodné hodnoty.\n"
      "\n"
      "  => o: %p\n"
      "  => *o: %s\n"
      "\n"
      , o
      , objektNaString(str, *o)
    );
  }

  if (navratNeboSekce("Inicializace dynamicky alokovaného objektu")) return;
  if (pokracovat())
  {
    *o = (Tobjekt) { .nazev="obj34", .stred={1, 2}, .vyska=3, .sirka=4};

    printh(
      "- Lze použít inicializaci anonymním objektem, ale je potřeba použít dereferenční\n"
      "  operátor " KK("*") ":\n"
      "\n"
      KOD(
      "  *o = (Tobjekt) { .nazev=\"obj34\", .stred={0, 0}, .vyska=1, .sirka=1};\n"
      )
      "\n"
      "  => o: %p\n"
      "  => *o: %s\n"
      "\n"
      , o
      , objektNaString(str, *o)
    );

    o->stred.x = -666;
    strcpy(o->nazev, "pentagram");

    printh(
      "- Stále jde použít inicializaci po složkách přes šipkový operátor.\n"
      "- Pozor u položek typu pole (zde textový řetězec). Jejich inicializaci\n"
      "  nelze provést přiřazením, musí se použít funkce (zde " KK("strcpy") ").\n"
      // Kopie polí přiřazením není v C možná, protože si typ pole obecně nemá kde
      // zapamatovat jeho délku. Když je pole, či textový řetězec (deklarovaný
      // jako pole) položkou struktury, jde ho přiřadit inicializátorem. Jakto?
      // Délku pole (což je údaj nutný pro správné provedení kopie) si nepamatuje
      // samotná položka typu pole, ale samotný typ struktura. Proto struktury
      // kopírovat lze, kdežto pole (mimo strukturu) ne.
      // Problém také nastává, když je pole deklarováno jako ukazatel. Pak při
      // kopii struktury dojde ke kopii ukazatele, ale ne samotného pole - provádí
      // se tzv. mělká kopie. Při práci s takovými strukturami je na to třeba
      // pamatovat.
      "\n"
      KOD(
      "  o->stred.x = -666;\n"
      "  strcpy(o->nazev, \"pentagram\");\n"
      )
      "\n"
      "  => o: %p\n"
      "  => *o: %s\n"
      "\n"
      , o
      , objektNaString(str, *o)
    );

    free(o);
  }

  if (navratNeboSekce("Výroba a inicializace pomocí funkce")) return;
  if (pokracovat())
  {
    o = novyObjekt("elipsa", 22, 33, 60, 20);
    if (o == NULL)
    {
      printf("Alokace se nepovedla.\n");
      return;
    }

    printh(
      "- Obvyklý způsob výroby dynamicky alokovaných objektů je pomocí funkce.\n"
      "- Můžeme jí říkat konstruktor.\n"
      "- Funkce zároveň dynamicky alokuje potřebný prostor a rovnou inicializuje\n"
      "  jednotlivé položky zadanými hodnotami.\n"
      "- Pokud se alokace nepovede, vrací NULL (je to potřeba testovat).\n"
      "\n"
      KOD(
      "  Tobjekt * novyObjekt(char nazev[], int x, int y, int sirka, int vyska);\n"
      )
      "\n"
      "- Použití konstruktoru vypadá takto:\n"
      "\n"
      KOD(
      "  o = novyObjekt(\"elipsa\", 22, 33, 60, 20);\n"
      "  if (o == NULL) return;\n"
      )
      "\n"
      "  => o: %p\n"
      "  => *o: %s\n"
      "\n"
      "- Výhodou tohoto řešení je, že funkce může při inicializaci provádět\n"
      "  i složitější věci než jen pouhé nastavování hodnot položek objektu.\n"
      "- Další výhodou je, že můžeme vyrobit různé konstruktory, které budou\n"
      "  vyrábět objekt na základě různě zadaných vstupních dat.\n"
      "\n"
      , o
      , objektNaString(str, *o)
    );
  }

  free(o);
//  o->sirka = 0; // Havárie!!

  if (navratNeboSekce("Uvolnění paměti")) return;
  if (pokracovat())
  {
    printh(
      "- Objekt alokovaný pomocí " KK("malloc") " musí být uvolněn pomocí funkce " KK("free") ".\n"
      "- Ideální je provést to ve stejném bloku kódu, ve kterém se provedla\n"
      "  alokace.\n"
      "\n"
      KOD(
      "  free(o);\n"
      "  o->sirka = 0; // Havárie!!\n"
      )
      "\n"
      "- Po uvolnění obsahuje proměnná " KK("o") " adresu do nealokované paměti!\n"
      "- Pokud hrozí, že by mohla být dále používána, je lepší ji nastavit na\n"
      "  hodnotu " KK("NULL") " (kterou jde alespoň testovat).\n"
      "\n"
      "- U složitějších objektů odkazujících na další dynamické proměnné, se\n"
      "  dealokace dělá vlastním podprogramem, který se o to postará.\n"
      "- Takovému podprogramu můžeme říkat destruktor.\n"
    );
  }

  konecKapitoly(__func__);
}


////////////////////////////////////////////////////////////////////////////////

/** Vytiskne objekt na výstup - objekt předávaný hodnotou.
 *
 * \param vystup FILE* Výstupní soubor nebo stream.
 * \param o Tobjekt Objekt předávaný hodnotou.
 * \return int Vrací počet znaků, které se vytiskly.
 */
int tiskniVerze1(FILE *vystup, Tobjekt o)
{
  return fprintf(vystup,
    "%s, %.2f, %.2f, %.2f, %.2f\n"
    , o.nazev, o.stred.x, o.stred.y, o.vyska, o.sirka
  );
}

/** Vytiskne objekt na výstup - objekt předávaný odkazem.
 *
 * \param vystup FILE* Výstupní soubor nebo stream.
 * \param o Tobjekt* Objekt předávaný odkazem.
 * \return int Vrací počet znaků, které se vytiskly.
 */
int tiskniVerze2(FILE *vystup, Tobjekt *o)
{
  //strcpy(o->nazev, "!!zmena!!"); // Funkce může měnit vnitřek objektu!
  //o->sirka = 666;                // Funkce může měnit vnitřek objektu!
  return fprintf(vystup,
    "%s, %.2f, %.2f, %.2f, %.2f\n"
    , o->nazev, o->stred.x, o->stred.y, o->vyska, o->sirka
  );
}

/** Vytiskne objekt na výstup - objekt předávaný odkazem na konstantní objekt.
 *
 * \param vystup FILE* Výstupní soubor nebo stream.
 * \param o const Tobjekt* Objekt předávaný odkazem na konstantní objekt.
 * \return int Vrací počet znaků, které se vytiskly.
 */
int tiskniVerze3(FILE *vystup, const Tobjekt *o)
{
  //strcpy(o->nazev, "!!zmena!!"); // Tohle způsobí varování překladače.
  //o->sirka = 666;                // Tohle je teď syntaktická chyba.
  return fprintf(vystup,
    "%s, %.2f, %.2f, %.2f, %.2f\n"
    , o->nazev, o->stred.x, o->stred.y, o->vyska, o->sirka
  );
}


void parametrFunkce(void)
{
  Tobjekt a = {.nazev="objektA", .stred={5, 5}, .sirka=55, .vyska=66};
  Tobjekt *b = novyObjekt("objektB", 10, 10, 20, 30);

  zacatekKapitoly("Struktura jako parametr funkce");

  printh(
    "- Objekty nechceme rutinně zpracovávat ručně, po položkách.\n"
    "- Ideální je mít ke každému typu struktura množinu funkcí, které budou\n"
    "  s objekty provádět základní, často používané operace.\n"
    "- Vytváření algoritmů je pak pohodlnější a bezpečnější.\n"
    "\n"
    "\n"
  );

  if (navratNeboSekce("Parametr předávaný hodnotou")) return;
  if (pokracovat())
  {
    printh(
      "- Při předávání objektů hodnotou dojde během volání funkce k (mělké) kopii\n"
      "  všech položek.\n"
      "- Pokud je struktura objemná (v praxi často kilobajty dat), vede to\n"
      "  k výraznému zpomalení programu.\n"
      "\n"
      KOD(
      "int tiskniVerze1(FILE *vystup, Tobjekt o)\n"
      "{\n"
      "  return fprintf(vystup, \n"
      "    \"%%s, %%.2f, %%.2f, %%.2f, %%.2f\\n\"\n"
      "    , o.nazev, o.stred.x, o.stred.y, o.vyska, o.sirka\n"
      "  );\n"
      "}\n"
      "...\n"
      "  Tobjekt a = {.nazev=\"objektA\", .stred={5, 5}, .sirka=55, .vyska=66};\n"
      "  Tobjekt *b = novyObjekt(\"objektB\", 10, 10, 20, 30);\n"
      "  tiskniVerze1(stdout, a);\n"
      "  tiskniVerze1(stdout, *b);\n"
      )
      "=>\n"
    );

    tiskniVerze1(stdout, a);
    tiskniVerze1(stdout, *b);

    printh(
      "\n"
      "- Objekty vyráběné dynamicky se v tomto případě musí předávat pomocí\n"
      "  dereferenčního operátoru " KK("*") ".\n"
      "- Tato varianta je syntakticky pohodlnější při používání lokálních\n"
      "  proměnných.\n"
      "- Při volání může být tento kód zbytečně pomalý.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Parametr předávaný odkazem")) return;
  if (pokracovat())
  {
    printh(
      "- Uvnitř funkce se místo tečkového používá šipkový operátor.\n"
      "- Syntakticky to je tedy stejně jednoduché jako při předávání hodnotou.\n"
      "- Při volání funkce se argument nekopíruje, pouze se předá odkaz na\n"
      "  původní objekt.\n"
      "- Funkce pracuje s původním objektem a může měnit jeho vnitřní položky.\n"
      "\n"
      KOD(
      "int tiskniVerze2(FILE *vystup, Tobjekt *o)\n"
      "{\n"
      "  //strcpy(o->nazev, \"!!zmena!!\"); // Funkce může měnit vnitřek objektu!\n"
      "  //o->sirka = 666;                // Funkce může měnit vnitřek objektu!\n"
      "  return fprintf(vystup, \n"
      "    \"%%s, %%.2f, %%.2f, %%.2f, %%.2f\\n\"\n"
      "    , o->nazev, o->stred.x, o->stred.y, o->vyska, o->sirka\n"
      "  );\n"
      "}\n"
      "...\n"
      "  tiskniVerze2(stdout, &a);\n"
      "  tiskniVerze2(stdout, b);\n"
      )
      "=>\n"
    );

    tiskniVerze2(stdout, &a);
    tiskniVerze2(stdout, b);

    printh(
      "\n"
      "- Lokální proměnné (resp. nikoli ukazatele) je při volání funkce potřeba\n"
      "  předávat pomocí referenčního operátoru " KK("&") ".\n"
      "- Tato varianta je syntakticky pohodlnější při používání dynamicky\n"
      "  alokovaných proměnných a obecně ukazatelů.\n"
      "- Je potřeba počítat s tím, že funkce teď může změnit hodnoty vnitřních\n"
      "  položek objektu.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Parametr typu odkaz na konstantní strukturu")) return;
  if (pokracovat())
  {
    printh(
      "- Někdy funkcí nechceme měnit položky objektu, ale chceme jej předávat\n"
      "  odkazem (např. kvůli pohodlí a efektivitě).\n"
      "- Potom lze parametr typu ukazatel označit klíčovým slovem " KK("const") ".\n"
      "- Funguje to jako jakýsi závazek, že funkce objekt nezmění.\n"
      "- Překladač pak bude sám hlídat, zda se funkce nepokouší změnit některou\n"
      "  z vnitřních položek objektu.\n"
      "\n"
      KOD(
      "int tiskniVerze3(FILE *vystup, const Tobjekt *o)\n"
      "{\n"
      "  //strcpy(o->nazev, \"!!zmena!!\"); // Tohle způsobí varování překladače.\n"
      "  //o->sirka = 666;                // Tohle je teď syntaktická chyba.\n"
      "  return fprintf(vystup, \n"
      "    \"%%s, %%.2f, %%.2f, %%.2f, %%.2f\\n\"\n"
      "    , o->nazev, o->stred.x, o->stred.y, o->vyska, o->sirka\n"
      "  );\n"
      "}\n"
      "\n"
      "  tiskniVerze3(stdout, &a);\n"
      "  tiskniVerze3(stdout, b);\n"
      )
      "\n"
      "=>\n"
    );

    tiskniVerze3(stdout, &a);
    tiskniVerze3(stdout, b);

    printh(
      "\n"
      "- Proč změna položky " KK("nazev") " způsobí jen varování a ne chybu při překladu?\n"
      "- Protože neměníme přímo položku " KK("nazev") ", což je ukazatel na začátek pole,\n"
      "  ale prvky tohoto pole.\n"
      "- Kdybychom měnili samotnou adresu, byla by to syntaktická chyba.\n"
      "- Stejně je syntaktickou chybou pokus o změnu položky " KK("sirka") ".\n"
    );
  }

  free(b);

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////

/** Nastaví hodnoty pro objekt - šestiúhelník.
 *
 * \param nazev char* Název objektu
 * \param x float Souřadnice středu
 * \param y float Souřadnice středu
 * \param strana float Délka strany šestiúhelníku
 * \return Tobjekt Vrací inicializovaný objekt.
 */
Tobjekt nastavSestiuhelnik(char *nazev, float x, float y, float strana)
{
  Tobjekt o = {
    .stred ={x, y},
    .sirka = 2*strana,
    .vyska = sqrt(3.0)*strana,
  };
  strcpy(o.nazev, nazev);
  return o;
}

/** Vyrobí a nastaví nový dynamický objekt - pětiúhelník.
 *
 * \param nazev char* Název objektu
 * \param x float Souřadnice středu
 * \param y float Souřadnice středu
 * \param strana float Délka strany pětiúhelníku
 * \return Tobjekt * Vrací ukazatel na alokovaný a inicializovaný objekt nebo NULL.
 */
Tobjekt * novyPetiuhelnik(char *nazev, float x, float y, float strana)
{
  Tobjekt * o = malloc(sizeof(Tobjekt));
  if (o != NULL)
  {
    o->stred.x = x;
    o->stred.y = y;
    o->sirka = strana*(1+sqrt(5))/2.0f;
    o->vyska = strana*sqrt(5 + 2*sqrt(5))/2.0f;
    strcpy(o->nazev, nazev);
  };
  return o;
}


void navratovaHodnotaFunkce(void)
{
  char str[100];

  zacatekKapitoly("Struktura jako návratová hodnota funkce");

  printh(
    "- Struktura může být návratovým typem funkce.\n"
    "- Vzhledem k rozměrům proměnných je ale častější, že funkce vracejí\n"
    "  ukazatel na dynamicky vyrobený objekt.\n"
    "\n"
  );

  if (navratNeboSekce("Funkce vracející hodnotu typu struktura")) return;
  if (pokracovat())
  {
    Tobjekt s = nastavSestiuhelnik("sestiuhelnik", 3, 3, 10);

    printh(
      "- Funguje v novějších verzích jazyka C (C99, C11, ...).\n"
      "- Funguje díky tomu, že nad strukturami funguje přiřazovací operátor,\n"
      "  proto lze výsledek funkce přiřadit do proměnné.\n"
      "- Dochází při tom k mělké kopii všech položek.\n"
      "- U objemných struktur to kvůli tomu bude pomalé.\n"
      "\n"
      "- Hodí se to pro inicializaci malých lokálních proměnných.\n"
      "- Oproti použití inicializátoru je to obecnější - umožňuje to při inicializaci\n"
      "  konzistentně provádět algoritmické výpočty.\n"
      "\n"
      KOD(
      "Tobjekt nastavSestiuhelnik(char *nazev, float x, float y, float strana)\n"
      "{\n"
      "  Tobjekt o = {\n"
      "    .stred ={x, y},\n"
      "    .sirka = 2*strana,\n"
      "    .vyska = sqrt(3.0)*strana,\n"
      "  };\n"
      "  strcpy(o.nazev, nazev);\n"
      "  return o;\n"
      "}\n"
      "...\n"
      "  Tobjekt s = nastavSestiuhelnik(\"sestiuhelnik\", 3, 3, 10);\n"
      )
      "\n"
      "=> s: %s\n"
      "\n"
      , objektNaString(str, s)
    );
  }


  if (navratNeboSekce("Funkce vracející dynamicky alokovaný objekt (ukazatel na něj)")) return;
  if (pokracovat())
  {
    Tobjekt * p = novyPetiuhelnik("pentagram", 2.5, 3.5, 10);

    printh(
      "- Funkce může vracet ukazatel na typ struktura.\n"
      "- Nesmí ale vracet ukazatel na lokální proměnnou, protože ta na konci\n"
      "  funkce zanikne a funkce by vrátila neplatný ukazatel.\n"
      "- Funkce může vracet ukazatel na dynamicky alokovaný objekt.\n"
      "\n"
      "- Hodí se to pro vytváření konstruktorů dynamických objektů.\n"
      "\n"
      KOD(
      "Tobjekt * novyPetiuhelnik(char *nazev, float x, float y, float strana)\n"
      "{\n"
      "  Tobjekt * o = malloc(sizeof(Tobjekt));\n"
      "  if (o != NULL)\n"
      "  {\n"
      "    o->stred.x = x;\n"
      "    o->stred.y = y;\n"
      "    o->sirka = strana*(1+sqrt(5))/2.0f;\n"
      "    o->vyska = strana*sqrt(5 + 2*sqrt(5))/2.0f;\n"
      "    strcpy(o->nazev, nazev); \n"
      "  };\n"
      "  return o;\n"
      "}\n"
      "...\n"
      "  Tobjekt * p = novyPetiuhelnik(\"pentagram\", 2.5, 3.5, 10);\n"
      )
      "\n"
      "=> *p: %s\n"
      "\n"
      , objektNaString(str, *p)
    );

    free(p);
  }

  konecKapitoly(__func__);
}







////////////////////////////////////////////////////////////////////////////////
void strukturaSUkazatelemNaObjekt(void)
{
  zacatekKapitoly("Struktura s ukazatelem na dynamicky alokovaný objekt");

  printh(
    "- Položkou typu struktura může být i ukazatel na jiný objekt.\n"
    "- Typickým využitím je položka s ukazatelem na dynamicky alokovaný objekt\n"
    "  stejného typu.\n"
    "- Objekt pak obsahuje ukazatel na jiný objekt stejného typu, ten na další\n"
    "  a tak dále. Poslední objekt obsahuje ukazatel " KK("NULL") ".\n"
    "- Objekty pak tvoří zřetězené dynamické struktury, jako například\n"
    "-- lineární seznam, zásobník, frontu, \n"
    "-- binární, ternární, či obecný strom,\n"
    "-- obecné grafové struktury.\n"
    "\n"
    "+---+---+   +---+---+   +---+---+   +---+---+\n"
    "| 1 | --|-->| 2 | --|-->| 3 | --|-->| 4 | --|--|\n"
    "+---+---+   +---+---+   +---+---+   +---+---+\n"
    "\n"
  );

  if (navratNeboSekce("Konstrukce typu s ukazatelem na objekt stejného typu")) return;
  if (pokracovat())
  {
    typedef struct _prvek Tprvek;

    struct _prvek
    {
      int data; // obecně libovolné další položky různých typů nebo typ Tdata...
      Tprvek * dalsi; // ukazatel na jiný objekt stejného typu
    };


    printh(
      "- Pro tento typ konstrukce se hodí rozdělená deklarace typu.\n"
      "- Deklarace ukazatele na další objekt je pak jednodušší.\n"
      "\n"
      KOD(
      "  typedef struct _prvek Tprvek;\n"
      "  \n"
      "  struct _prvek\n"
      "  {\n"
      "    int data; // obecně libovolné další položky různých typů nebo typ Tdata...\n"
      "    Tprvek * dalsi; // ukazatel na jiný objekt stejného typu\n"
      "  };\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Typ s ukazatelem na stejný typ - starší varianta")) return;
  if (pokracovat())
  {
//    typedef struct _prvek
//    {
//      int data; // obecně libovolné další položky různých typů nebo typ Tdata...
//      struct _prvek * dalsi; // Zde ještě typ Tprvek neexistuje, proto takto.
//    } Tprvek;

    printh(
      "- Dá se použít i nedělená deklarace typu.\n"
      "- Deklarace ukazatele na další objekt ale vyžaduje použití dvouslovného\n"
      "  názvu typu s klíčovým slovem " KK("struct") ", protože v místě deklarace této\n"
      "  položky ještě neexistuje nově vytvářený název typu.\n"
      "\n"
      KOD(
      "  typedef struct _prvek\n"
      "  {\n"
      "    int data; // obecně libovolné další položky různých typů nebo typ Tdata...\n"
      "    struct _prvek * dalsi; // Zde ještě typ Tprvek neexistuje, proto takto.\n"
      "  } Tprvek;\n"
      )
    );
  }

  konecKapitoly(__func__);
}



////////////////////////////////////////////////////////////////////////////////

void napoveda(void)
{
  printf("Toto je napoveda k programu...\n");
}


void strukturaSUkazatelemNaFunkci(void)
{
  /* Struktura s ukazatelem na funkci sice vytvořit jde, ale nedá se to moc
  používat jako v objektově orientovaných jazycích. Pořád je potřeba předávat
  parametr s ukazatelem na objekt, protože funkce k němu nemá žádný automatický
  přístup.
  Dá se to ale použít třeba pro uložení specifické akce, například může být
  objekt položkou menu a funkce uložená v objektu je akce, kterou je třeba
  vykonat při výběru dané položky.
  */

  zacatekKapitoly("Struktura s ukazatelem na funkci");

  printh(
    "- Jazyk C umí pracovat s typem ukazatel na funkci.\n"
    "- Proměnná takového typu může být také položkou struktury.\n"
    "- Na rozdíl od objektově orientovaných jazyků nemá taková funkce\n"
    "  automaticky přístup k prvkům struktury.\n"
    "- I tak se to dá využít pro různé situace.\n"
    "\n"
    "\n"
  );

  if (navratNeboSekce("Ukázkový příklad")) return;
  if (pokracovat())
  {
    printh(
      "- Vytvoříme položku menu s textovou nabídkou.\n"
      "- Každá položka menu si bude pamatovat spouštěcí klávesu, na kterou\n"
      "  má reagovat, dále textovou nabídku a obslužnou funkci, která\n"
      "  se vyvolá, když bude příslušná nabídka vybrána.\n"
      "\n"
    );
  }

  typedef void (* TFobsluha)(void);

  if (navratNeboSekce("Typ ukazatel na funkci")) return;
  if (pokracovat())
  {
    printh(
      "- Typ jde vytvořit pro funkce s přesně odpovídající signaturou.\n"
      "  Kompatibilní funkce pak musí mít stejný návratový typ, stejný počet\n"
      "  parametrů stejných typů.\n"
      "- Zde je ukázka nejjednoduššího typu ukazatel na funkci bez parametrů,\n"
      "  která nic nevrací (tj. má návratový typ void):\n"
      "\n"
      KOD(
      "  typedef void (* TFobsluha)(void);\n"
      )
      "\n"
      "- Jazyk C umí pracovat pouze s typem ukazatel na funkci. Nejde vytvořit\n"
      "  typ \"funkce\".\n"
      "\n"
    );
  }

  typedef struct
  {
    int klavesa;
    const char * text;
    TFobsluha obsluha;
  } Tnabidka;

  if (navratNeboSekce("Struktura s ukazatelem na funkci")) return;
  if (pokracovat())
  {
    printh(
      "- Typ pro popis jednoho řádku menu může vypadat takto:\n"
      "\n"
      KOD(
      "  typedef struct\n"
      "  {\n"
      "    int klavesa;\n"
      "    const char * text;\n"
      "    TFobsluha obsluha;\n"
      "  } Tnabidka;\n"
      )
      "\n"
      "- Položka " KK("obsluha") " je typu " KK("TFobsluha") ", nikoli typu " KK("TFobsluha *") ", protože\n"
      "  tento typ už je sám o sobě ukazatelem.\n"
      "\n"
    );
  }

  Tnabidka n = { .klavesa='N', .text="Vypis napovedu", .obsluha=napoveda };

  if (navratNeboSekce("Inicializace objektu s ukazatelem na funkci")) return;
  if (pokracovat())
  {
    printh(
      "- Do položky " KK("obsluha") " nastavíme ukazatel na konkrétní funkci jednoduše\n"
      "  přiřazením jména kompatibilní funkce.\n"
      "- Jméno funkce bez kulatých závorek (operátor volání funkce) je ukazatelem\n"
      "  na tuto funkci.\n"
      "- Pro náš ukázkový příklad použiji nabídku menu pro výpis nápovědy,\n"
      "  jejíž obslužná bude vypisovat nápovědu k programu.\n"
      "- V reálném programu mohou být jednotlivé nabídky menu poskládány\n"
      "  například do pole nebo lineárního seznamu.\n"
      "\n"
      KOD(
      "void napoveda(void)\n"
      "{\n"
      "  printf(\"Toto je napoveda k programu...\\n\");\n"
      "}\n"
      "...\n"
      "  Tnabidka n = { .klavesa=\'N\', .text=\"Vypis napovedu\", .obsluha=napoveda };\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Zavolání obslužné funkce")) return;
  if (pokracovat())
  {
    printh(
      "- S inicializovanou proměnnou typu ukazatel na funkci se dále pracuje\n"
      "  jako s obyčejnou funkcí.\n"
      "- Funkce se zavolá pomocí kulatých závorek.\n"
      "\n"
      KOD(
      "  n.obsluha();\n"
      )
      "\n"
      "=> "
    );

    n.obsluha();

    printh(
      "\n"
      "- V reálném programu samozřejmě bude kolem typu " KK("TFobsluha") " vytvořeno\n"
      "  několik funkcí, které umožní snadno přidávat položky menu, tisknout menu\n"
      "  na obrazovku, vyhledávat v menu klávesu zadanou uživatelem a vyvolávat\n"
      "  zaregistrovanou obsluhu vybrané položky menu.\n"
      "- Tyto funkce nutně nebudou dalšími položkami struktury. Spíše půjde\n"
      "  o funkce, které budou samy zpracovávat objekty s reprezentací samotného\n"
      "  menu. (Viz další kapitolu.)\n"
      "\n"
    );
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////
void zakladOOP(void)
{
  zacatekKapitoly("Struktura jako základ objektově orientovaného programování");

  printh(
    "- Jazyk C sice nepodporuje dědičnost a jiné vlastnosti známé z OOP,\n"
    "  ale pomocí struktur se lze některým rysům OOP přiblížit.\n"
    "- Základem je znalost koncepce abstraktního datového typu (ADT).\n"
    "\n"
  );

  if (navratNeboSekce("Abstraktní datový typ (ADT)")) return;
  if (pokracovat())
  {
    printh(
      "- Každý datový typ je definován třemi věcmi:\n"
      "  1. svým jedinečným názvem,\n"
      "  2. počtem a tvarem hodnot, kterých mohou nabývat proměnné,\n"
      "  3. množinou operací, které lze nad proměnnými provádět.\n"
      "\n"
      "- U zabudovaných typů jsou tyto věci zabudovány v překladači.\n"
      "- U uživatelských typů můžeme/musíme tyto tři věci realizovat sami.\n"
      "\n"
      "- ADT vytvořený pomocí struktury a množina funkcí, které objekty tohoto\n"
      "  typu zpracovávají, jsou esencí tzv. " BB("strukturovaného programování") ".\n"
      "- Zároveň je tento koncept přímým předchůdcem objektově orientovaného\n"
      "  programování - konkrétně jde o koncept zapouzdření.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Praktické zásady při návrhu ADT")) return;
  if (pokracovat())
  {
    printh(
      "- ADT má reprezentovat či modelovat jednu entitu reálného světa nebo programu.\n"
      "-- Příklady: položka menu, menu, seznam položek, auto, postava ve hře, ...\n"
      "\n"
      "- Uživatel by neměl mít potřebu přístupu k vnitřním položkám objektů.\n"
      "- ADT je vhodné zabalit do samostatného modulu a v hlavičkovém souboru\n"
      "  zveřejnit jen funkce, které jsou skutečně potřebné pro používání ADT.\n"
      "  V modulu samotném mohou být i neveřejné funkce, které realizují části\n"
      "  algoritmů, ale nepatří do rukou koncovému uživateli.\n"
      "- V modulu se dají ukrýt i vnitřní položky datového typu struktura.\n"
      "\n"
      "- Musí existovat všechny základní funkce pro práci s objekty a jejich\n"
      "  vnitřními položkami tak, aby si uživatel ADT mohl složitější činnost\n"
      "  poskládat z těchto základních operací.\n"
      "- V jednoduchosti je krása. Příliš velké množství funkcí znamená, že\n"
      "  je uživatel asi nebude umět správně použít.\n"
      "\n"
      "- Neboj se dynamicky alokovaných objektů. Pracují-li úplně všechny funkce\n"
      "  s dynamicky alokovanými objekty (a tedy používají ukazatele na ně),\n"
      "  ve výsledku to znamená méně hvězdiček a ampersandů (*, &) v běžném kódu.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka: ADT Menu")) return;
  if (pokracovat())
  {
    printh(
      "- Prostuduj si zdrojové soubory tohoto projektu menu.h a menu.c.\n"
      "- Je v nich realizován ADT Menu, který je pak použit ve funkci " KK("main") "\n"
      "  tohoto programu a realizuje jednoduchou textovou nabídku.\n"
      "\n"
      PODSEKCE("Položka menu")
      "- Základem menu je typ popisující jednu položku nabídky:\n"
      "\n"
      KOD(
      "typedef struct _polozka Tpolozka;\n"
      "\n"
      "struct _polozka\n"
      "{\n"
      "  char aktivator;        /**< Aktivační znak nabídky */\n"
      "  // const je závazek, že se text nebude modifikovat\n"
      "  const char * nabidka;  /**< Text nabídky */\n"
      "  TFobsluha akce;        /**< Ukazatel na funkci */\n"
      "  Tpolozka * dalsi;      /**< Ukazatel na další položku menu. */\n"
      "};\n"
      )
      "\n"
      "- Prvky s nabídkami tvoří lineární seznam - viz položku " KK("dalsi") ".\n"
      "- Tento typ je pro koncového uživatele ukrytý - nepotřebuje jej vidět.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Typ zastřešující celé menu")) return;
  if (pokracovat())
  {
    printh(
      "- Typem pro uživatele je " KK("Tmenu *") ". Samotná deklarace typu " KK("Tmenu") " je rozdělená\n"
      "  mezi hlavičkový a zdrojový soubor, takže uživatel menu nevidí dovnitř\n"
      "  a musí s typem pracovat přes ukazatel.\n"
      "- Typ " KK("Tmenu") " je vlastně zastřešující strukturou realizující lineární seznam.\n"
      "  Obsahuje ukazatel na první a poslední prvek seznamu, plus hlavičku menu.\n"
      "\n"
      PODSEKCE("V souboru menu.h:")
      "\n"
      KOD(
      "typedef struct _menu Tmenu;\n"
      )
      "\n"
      PODSEKCE("V souboru menu.c:")
      "\n"
      KOD(
      "struct _menu\n"
      "{\n"
      "  const char *hlavicka;\n"
      "  Tpolozka * prvni;\n"
      "  Tpolozka * posledni;\n"
      "};\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Operace ADT Menu")) return;
  if (pokracovat())
  {
    printh(
      "- Pro práci s typem " KK("Tmenu *") " nabízí modul 4 funkce, které uživateli\n"
      "  poskytují vše potřebné pro realizaci menu a práci s ním.\n"
      "\n"
      "- Funkce " KK("menuVytvor") " a " KK("menuZrus") " realizují konstruktor a destruktor\n"
      "  celého menu.\n"
      "- První funkce dynamicky alokuje prázdné menu, do kterého je potřeba přidat\n"
      "  položky s nabídkami. Druhá pak uvolní z paměti celý lineární seznam\n"
      "  položek i samotné menu.\n"
      "\n"
      KOD(
      "Tmenu * menuVytvor(const char *hlavicka);\n"
      "void menuZrus(Tmenu * menu);\n"
      )
      "\n"
      "- Další funkcí je " KK("menuPridej") ", která vytvoří novou položku menu a zařadí\n"
      "  ji na konec lineárního seznamu.\n"
      "\n"
      KOD(
      "bool menuPridej(Tmenu * menu, char aktivator, const char * nabidka, TFobsluha akce);\n"
      )
      "\n"
      "- Poslední funkcí je " KK("menuVolba") ", která realizuje zobrazení menu\n"
      "  na obrazovce, čekání na volbu od uživatele a posléze vykonání akce\n"
      "  zaregistrované ke zvolené nabídce.\n"
      "\n"
      KOD(
      "int menuVolba(Tmenu *menu);\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Realizace a použití menu")) return;
  if (pokracovat())
  {
    printh(
      "- Po vytvoření proměnné pro menu je potřeba vložit jednotlivé položky menu.\n"
      "- Například tato kapitola je do menu vložena takto:\n"
      "\n"
      KOD(
      "  menuPridej(menu, \'L\', \"Struktura jako základ OOP\\n\", zakladOOP);\n"
      )
      "\n"
      "- Proměnná menu byla vytvořena pomocí funkce " KK("menuVytvor") ".\n"
      "- Znak \'L\' je aktivační znak, na který čeká funkce " KK("menuVolba") ".\n"
      "- Hodnota " KK("zakladOOP") " je ukazatel na funkci realizující výpis této kapitoly.\n"
      "\n"
      "- Samotné vypisování a fungování menu realizuje cyklus, ve kterém se opakovaně\n"
      "  volá funkce " KK("menuVolba") ".\n"
      "\n"
      KOD(
      "  while (menuVolba(menu) != 'Q') {}\n"
      )
      "\n"
    );
  }

  konecKapitoly(__func__);
}



////////////////////////////////////////////////////////////////////////////////
void typUnion(void)
{
  zacatekKapitoly("Datový typ union");

  printh(
    "- Syntakticky vypadá po všech stránkách stejně, jako typ " KK("struct") ".\n"
    "- Místo klíčového slova " KK("struct") " se používá " KK("union") ".\n"
    "- Naprosto stejně se zapisují i položky a stejně se k nim přistupuje.\n"
    "\n"
    "- Zásadní rozdíl je ve způsobu uložení položek.\n"
    "- Všechny položky v unii sdílejí stejný prostor, tj. jako kdyby byly uloženy\n"
    "  přes sebe.\n"
    "- Všechny položky unie mají svůj začátek na stejné adrese.\n"
    "- Velikost unie (" KK("sizeof") ") je dána velikostí největší položky.\n"
    "\n"
    "- Unie neslouží pro uložení různých položek v jedné proměnné.\n"
    "\n"
  );

  if (navratNeboSekce("Různý pohled na stejná data")) return;
  if (pokracovat())
  {
    typedef union {
      int cislo;
      unsigned char bajt[sizeof(int)];
    } TUhodnota;

    TUhodnota x;
    x.cislo = 0xAB12CD34;

    printh(
      "- Unie se dá použít jako nástroj pro zkoumání stejných dat optikou různých\n"
      "  datových typů.\n"
      "\n"
      KOD(
      "  typedef union {\n"
      "    int cislo;\n"
      "    unsigned char bajt[sizeof(int)];\n"
      "  } TUhodnota;\n"
      "\n"
      "  TUhodnota x;\n"
      "  x.cislo = 0xAB12CD34;\n"
      )
      "\n"
      "  => x.cislo: 0x%X => desítkově: %d\n"
      "  => x.bajt[0]: 0x%X, x.bajt[1]: 0x%X, x.bajt[2]: 0x%X, x.bajt[3]: 0x%X\n"
      "\n"
      "- Zde se na číslo typu " KK("int") " díváme zároveň jako na pole čtyř bajtů.\n"
      "\n"
      , x.cislo, x.cislo
      , x.bajt[0], x.bajt[1], x.bajt[2], x.bajt[3]
    );
  }


  if (navratNeboSekce("Vzájemně vyloučené části struktury")) return;
  if (pokracovat())
  {
    typedef struct {
      bool anonym;
      union {
        char jmeno[50+1];
        int kod;
      } id;
      int pocetBodu;
    } Tstudent;

    Tstudent a = { .anonym=true, .id.kod=865651, .pocetBodu=15 };
    Tstudent b = { .anonym=false, .id.jmeno="Alois Novak", .pocetBodu=105 };

    printh(
      "- Unie se dá použít také uvnitř struktury.\n"
      "- Pokud existují vzájemně vyloučené položky, mohou sdílet stejné místo.\n"
      "\n"
      "- Příklad: Máme zaznamenat známky studentů u online testu. Studenti\n"
      "  zadávají buď své jméno, nebo dělají test anonymně a identifikují se\n"
      "  číselným kódem. Struktura pro tuto situaci může vypadat takto:\n"
      "\n"
      KOD(
      "  typedef struct {\n"
      "    bool anonym;\n"
      "    union {\n"
      "      char jmeno[50+1];\n"
      "      int kod;\n"
      "    } id;\n"
      "    int pocetBodu;\n"
      "  } Tstudent;\n"
      "\n"
      "  Tstudent a = { .anonym=true, .id.kod=865651, .pocetBodu=15 };\n"
      "  Tstudent b = { .anonym=false, .id.jmeno=\"Alois Novak\", .pocetBodu=105 };\n"
      )
      "\n"
      "=> a: { .id.kod=%d, .pocetBodu=%d }\n"
      "=> b: { .id.jmeno=%s, .pocetBodu=%d }\n"
      "\n"
      , a.id.kod, a.pocetBodu
      , b.id.jmeno, b.pocetBodu
    );
  }

  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////

#include <stdint.h>

void bitovePole(void)
{
  zacatekKapitoly("Bitové pole");

  printh(
    "- Ačkoli se to nazývá bitové pole (anglicky bit field), realizuje se to\n"
    "  pomocí struktury.\n"
    "- Pokud je položka celočíselného typu, jde předepsat, kolik bitů bude\n"
    "  pro ni použito.\n"
    "- Další položka stejného typu pak může využít vedlejší bity atd.\n"
    "\n"
    "- Umožňuje to pracovat\n"
    "  - s jednotlivými bity celočíselné hodnoty,\n"
    "  - s několikabitovými částmi celočíselné hodnoty.\n"
    "\n"
    "- Často se používá ve spojení s typem " KK("union") ".\n"
    "\n"
  );

  if (navratNeboSekce("Ukázka - zapakované datum")) return;
  if (pokracovat())
  {
    typedef struct {
      uint32_t den  : 5;  // 2^5 = 32 => rozsah 0..31 je akorát pro dny
      uint32_t mesic: 4;  // 2^4 = 16 => rozsah 0..15 je akorát pro měsíce
      uint32_t rok  : 23; // 2^23 => tohle pro uložení roků určitě stačí
    } Tdatum;

    Tdatum d = { .den=14, .mesic=8, .rok=2023 };

    printh(
      "\n"
      KOD(
      "  #include <stdint.h>\n"
      "  typedef struct {\n"
      "    uint32_t den  : 5;  // 2^5 = 32 => rozsah 0..31 je akorát pro dny\n"
      "    uint32_t mesic: 4;  // 2^4 = 16 => rozsah 0..15 je akorát pro měsíce\n"
      "    uint32_t rok  : 23; // 2^23 => tohle pro uložení roků určitě stačí\n"
      "  } Tdatum;\n"
      "\n"
      "  Tdatum d = { .den=14, .mesic=8, .rok=2023 };\n"
      )
      "\n"
      "  => d.den: %d, d.mesic: %d, d.rok: %d\n"
      "  => sizeof(Tdatum): %zu bajty = %zu bitů\n"
      "\n"
      , d.den, d.mesic, d.rok
      , sizeof(Tdatum), sizeof(Tdatum)*8
    );

    printh(
      "- Cílem této ukázky je ušetřit místo a do prostoru pro jednu číselnou\n"
      "  proměnnou zakódovat více hodnot menšího rozsahu.\n"
      "- Běžně pro uložení data potřebujeme tři číselné proměnné pro uložení\n"
      "  dne, měsíce a roku.\n"
      "- Protože tyto hodnoty mají velmi malé rozsahy, můžeme je všechny vměstnat\n"
      "  do prostoru jediného čísla typu " KK("unsigned int") ".\n"
      // Pozor. Kdyby to byl int, i jednotlivé bitové kousky by byly znaménkové,
      // tj. nejvyšší bit by se bral jako znaménko.
      "- Pro uložení dne stačí 5 bitů, pro uložení měsíce 4 bity a zbytek do\n"
      "  32 bitů stačí pro uložení hodnoty roku.\n"
      "\n"
      "- Poznámka: Je zde použit typ s pevnou velikostí " KK("uint32_t") " ze " KK("stdint.h") ".\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka - bitové pole a union")) return;
  if (pokracovat())
  {
    typedef union {
      int32_t cislo;
      struct {
        uint32_t liche    : 1;  // první bit zprava určuje, lichost/sudost
        uint32_t bity     : 30; // zbývající bity
        uint32_t znamenko : 1;  // znaménkový bit - první bit zleva
      } hack;
    } Thack;

    Thack x = { .cislo = 17 };
    Thack y = { .cislo = -4 };

    /* Poznámka: Všimni si, že první položka odpovídá bitu nejvíce vpravo,
    kdežto znaménkový bit musí být až jako poslední položka.

    +17:
    00000000 00000000 00000000 00000000 00000000 00000000 00000000 00010001
    ^                                                                     ^
    znaménkový bit                                             nejnižší bit
    */

    printh(
      "\n"
      KOD(
      "  typedef union {\n"
      "    int32_t cislo;\n"
      "    struct {\n"
      "      uint32_t liche    : 1;  // první bit zprava určuje, lichost/sudost\n"
      "      uint32_t bity     : 30; // zbývající bity\n"
      "      uint32_t znamenko : 1;  // znaménkový bit - první bit zleva\n"
      "    } hack;\n"
      "  } Thack;\n"
      "\n"
      "  Thack x = { .cislo = 17 };\n"
      "  Thack y = { .cislo = -4 };\n"
      )
      "\n"
      "  => x.cislo: %d, x.hack.znamenko = %d, x.hack.liche = %d\n"
      "  => y.cislo: %d, y.hack.znamenko = %d, y.hack.liche = %d\n"
      "  => sizeof(Thack): %zu bajty = %zu bitů\n"
      "\n"
      , x.cislo, x.hack.znamenko, x.hack.liche
      , y.cislo, y.hack.znamenko, y.hack.liche
      , sizeof(Thack), sizeof(Thack)*8
    );
  }

  if (navratNeboSekce("Další poznámky k bitovým polím")) return;
  if (pokracovat())
  {
    printh(
      "- Na položky bitového pole nejde vytvořit ukazatel, protože mohou začínat\n"
      "  i uprostřed bajtu.\n"
      "- Norma jazyka C zaručuje, že přetečení jedné neovlivní okolní položky.\n"
      // Souvisí to s tím, že se před uložením vždy dělá maskování na daný počet
      // bitů. V GCC to tak funguje. V ostatních překladačích snad taky.
      "- Při práci s nimi se na pozadí dělají bitové operace jako maskování\n"
      "  a bitové posuny.\n"
      "  => Stojí to nějaký čas a je třeba zvážit, zda to za to stojí.\n"
      "\n"
      "- Použití v běžných algoritmech nedoporučuji. Nestojí to za to.\n"
      "- Určitou hodnotu to může mít při práci s binárními soubory nebo v aplikacích,\n"
      "  kde se počítá každý bajt.\n"
      // Arduino, přenosy po síti, vzduchem apod., pro komunikaci s modelem,
      // dronem atp.

      // Norma jazyka umožňuje i další kouzla, která jsem tady nezmínil, jako
      // třeba zarovnávání na nejbližší bajt, práce s little-big endian a jiné.
      // Kdo to potřebuje, nechť si to najde v manuálu jazyka.
      "\n"
    );
  }


  konecKapitoly(__func__);
}


